// Modified version by Kotilo. All modifications are under GPLv2. Original:
// Copyright (C) 2002-2005 Nikolaus Gebhardt
// This file is part of the "Irrlicht Engine" and the "irrXML" project.
// For conditions of distribution and use, see copyright notice in irrlicht.h and/or irrXML.h

#ifndef __ICXML_READER_IMPL_H_INCLUDED__
#define __ICXML_READER_IMPL_H_INCLUDED__

#include "irrXML.h"
#include "irrString.h"
#include "irrArray.h"
#include "fast_atof.h"

#ifdef _DEBUG
#define IRR_DEBUGPRINT(x) printf((x));
#else // _DEBUG 
#define IRR_DEBUGPRINT(x)
#endif // _DEBUG


//! implementation of the IrrXMLReader
class CXMLReaderImpl {
public:

	//! Constructor
	CXMLReaderImpl(FileReadCallBack* callback, bool deleteCallBack = true)    Seg4;

	//! Destructor
	~CXMLReaderImpl()        Seg4;

	//! Reads forward to the next xml node. 
	//! \return Returns false, if there was no further node. 
	bool read()    Seg4;

	//! Returns the type of the current XML node.
	EXML_NODE getNodeType() const     Seg4;

	//! Returns attribute count of the current XML node.
	int getAttributeCount() const     Seg4;

	//! Returns name of an attribute.
	const c8* getAttributeName(int idx) const    Seg4;

	//! Returns the value of an attribute. 
	const c8* getAttributeValue(int idx) const    Seg4;

	//! Returns the value of an attribute. 
	const c8* getAttributeValue(const c8* name) const   Seg4;

	//! Returns the value of an attribute
	const c8* getAttributeValueSafe(const c8* name) const   Seg4;


	//! Returns the value of an attribute as integer. 
	int getAttributeValueAsInt(const c8* name) const   Seg4;

	//! Returns the value of an attribute as integer. 
	int getAttributeValueAsInt(int idx) const   Seg4;

	//! Returns the value of an attribute as float. 
	float getAttributeValueAsFloat(const c8* name) const  Seg4;

	//! Returns the value of an attribute as float. 
	float getAttributeValueAsFloat(int idx) const  Seg4;

	//! Returns the name of the current node.
	const c8* getNodeName() const  Seg4;

	//! Returns data of the current node.
	const c8* getNodeData() const  Seg4;

	//! Returns if an element is an empty element, like <foo />
	bool isEmptyElement() const  Seg4;

	//! Returns format of the source xml file.
	ETEXT_FORMAT getSourceFormat() const Seg4;

	//! Returns format of the strings returned by the parser.
	ETEXT_FORMAT getParserFormat() const Seg4;

private:

	// Reads the current xml node
	void parseCurrentNode()   Seg4;

	//! sets the state that text was found. Returns true if set should be set
	bool setText(c8* start, c8* end) Seg4;

	//! ignores an xml definition like <?xml something />
	void ignoreDefinition() Seg4;

	//! parses a comment
	void parseComment() Seg4;

	//! parses an opening xml element and reads attributes
	void parseOpeningXMLElement() Seg4;

	//! parses an closing xml tag
	void parseClosingXMLElement() Seg4;

	//! parses a possible CDATA section, returns false if begin was not a CDATA section
	bool parseCDATA() Seg4;

	// finds a current attribute by name, returns 0 if not found
	const SAttribute* getAttributeByName(const c8* name) const Seg4;

	// replaces xml special characters in a string and creates a new one
	estring replaceSpecialCharacters(estring& origstr) Seg4;

	//! reads the xml file and converts it into the wanted character format.
	bool readBuffer(FileReadCallBack* callback) Seg4;

	//! returns if a format is little endian
	inline bool isLittleEndian(ETEXT_FORMAT f) Seg4;

	//! returns true if a character is whitespace
	inline bool isWhiteSpace(c8 c) Seg4;

	//! generates a list with xml special characters
	void createSpecialCharacterList() Seg4;

	//! compares the first n characters of the strings
	bool equalsn(const c8* str1, const c8* str2, int len) Seg4;

	//! stores the target text format
	void storeTargetFormat() Seg4;

	// instance variables:

	c8* TextData;         // data block of the text file
	c8* P;                // current point in text to parse
	c8* TextBegin;        // start of text to parse
	unsigned int TextSize;       // size of text to parse in characters, not bytes

	EXML_NODE CurrentNodeType;   // type of the currently parsed node
	ETEXT_FORMAT SourceFormat;   // source format of the xml file
	ETEXT_FORMAT TargetFormat;   // output format of this parser

	estring NodeName;    // name of the node currently in
	estring EmptyString; // empty string to be returned by getSafe() methods

	bool IsEmptyElement;       // is the currently parsed node empty?

	estring_array SpecialCharacters; // see createSpecialCharacterList()

	SAttribute_array Attributes; // attributes of current element
	
}; // end CXMLReaderImpl


#endif
