// Modified version by Kotilo. All modifications are under GPLv2. Original:
// Copyright (C) 2002-2005 Nikolaus Gebhardt
// This file is part of the "Irrlicht Engine" and the "irrXML" project.
// For conditions of distribution and use, see copyright notice in irrlicht.h and/or irrXML.h

#ifndef __IRR_XML_H_INCLUDED__
#define __IRR_XML_H_INCLUDED__

#include "irrTypes.h"
#include "segments.h"
#include <PalmOS.h>

	
//! Enumeration of all supported source text file formats 
enum ETEXT_FORMAT
{
	//! ASCII, file without byte order mark, or not a text file
	ETF_ASCII,

	//! UTF-8 format
	ETF_UTF8,

	//! UTF-16 format, big endian
	ETF_UTF16_BE,

	//! UTF-16 format, little endian
	ETF_UTF16_LE,

	//! UTF-32 format, big endian
	ETF_UTF32_BE,

	//! UTF-32 format, little endian
	ETF_UTF32_LE,
};


//! Enumeration for all xml nodes which are parsed by IrrXMLReader
enum EXML_NODE
{
	//! No xml node. This is usually the node if you did not read anything yet.
	EXN_NONE,

	//! A xml element, like <foo>
	EXN_ELEMENT,

	//! End of an xml element, like </foo>
	EXN_ELEMENT_END,

	//! Text within a xml element: <foo> this is the text. </foo>
	EXN_TEXT,

	//! An xml comment like &lt;!-- I am a comment --&gt; or a DTD definition.
	EXN_COMMENT,

	//! An xml cdata section like &lt;![CDATA[ this is some CDATA ]]&gt;
	EXN_CDATA,

	//! Unknown element.
	EXN_UNKNOWN
};
	

//! defines the utf-16 type.
/** Not using wchar_t for this because 
wchar_t has 16 bit on windows and 32 bit on other operating systems. */
typedef unsigned short char16;

//! defines the utf-32 type. 
/** Not using wchar_t for this because 
wchar_t has 16 bit on windows and 32 bit on other operating systems. */
typedef unsigned long char32;

	
//! Implementation of the file read callback for ordinary files
class FileReadCallBack {
public:
	FileReadCallBack(const char* fileName) Seg4;
	~FileReadCallBack() Seg4;
	
	//! Reads an amount of bytes from the file.
	int read(void* buffer, int sizeToRead) Seg4;

	//! Returns size of file in bytes
	int getSize() Seg4;

private:

	//! retrieves the file size of the open file
	void getFileSize() Seg4;
	
	FileHand m_fileH;
	Int32    m_size;
	bool     m_close;
}; 



#endif // __IRR_XML_H_INCLUDED__
