#include "timeUtils.h"

#include <string.h>
#include <DateTime.h>
#include <TimeMgr.h>
#include <StringMgr.h>
#include <MemoryMgr.h>


#define CURRENT_DATE_TIME_BUFFER_LEN 24
char currentDateTime[CURRENT_DATE_TIME_BUFFER_LEN];


const char* getCurrentDateTime()
{
	DateTimeType dateTime;
	UInt32 seconds = TimGetSeconds();
	TimSecondsToDateTime(seconds, &dateTime);
	
	// Write date and time in the form: 2009-06-25T09:19:00.000
	// That is: yyyy-mm-ddThh:mm:ss.sss
	// year-month-dayThours:minutes:seconds.hundreds_of_seconds.
	// This is the dateTime-format of XML Schema. Note that the hundreds of
	// seconds are currently always written as 000.		
	
	memset(currentDateTime, '\0', CURRENT_DATE_TIME_BUFFER_LEN);
		
	StrPrintF(currentDateTime, "%04d-%02d-%02dT%02d:%02d:%02d.000", dateTime.year,
																	dateTime.month,
																	dateTime.day,
																	dateTime.hour,
																	dateTime.minute,
																	dateTime.second);

	return currentDateTime;
}

UInt32 xmlTimeToSeconds(const char* xmlTimeString){
	char hours[3];
	char minutes[3];
	char seconds[3];
	
	MemMove(hours, xmlTimeString, 2);
	hours[2] = '\0';

	MemMove(minutes, xmlTimeString + 3, 2);
	minutes[2] = '\0';
	
	MemMove(seconds, xmlTimeString + 6, 2);
	seconds[2] = '\0';
	
	return (StrAToI(hours) * 3600 + StrAToI(minutes) * 60 + StrAToI(seconds));
}

UInt32 xmlDateTimeToSeconds(const char* xmlDateTimeString){
	DateTimeType time;
	char year[5];
	char month[3];
	char day[3];
	
	time.hour = 0;
	time.minute = 0;
	time.second = 0;
	
	MemMove(year, xmlDateTimeString, 4);
	year[4] = '\0';

	MemMove(month, xmlDateTimeString + 5, 2);
	month[2] = '\0';
	
	MemMove(day, xmlDateTimeString + 8, 2);
	day[2] = '\0';
	
	time.year = StrAToI(year);
	time.month = StrAToI(month);
	time.day = StrAToI(day);

	return (TimDateTimeToSeconds(&time) + xmlTimeToSeconds(xmlDateTimeString + 11));
}

void xmlToPalmDateTime(const char* timeString, DateTimeType* time){
	TimSecondsToDateTime(xmlDateTimeToSeconds(timeString), time);
}
