#include "AlarmTimes.h"
#include "../../Preferences.h"
#include <Preferences.h>
#include <DataMgr.h>
#include <MemoryMgr.h>
#include <DateTime.h>
#include <TimeMgr.h>
#include <SysUtils.h>

#define SECONDSINADAY 86400  
#define appFileCreator	'MIHA'

void bubleSort(UInt32* array, UInt32 size){
	UInt32 i, j, t;
	for(i = 1; i < size; i++){
		j = i;
		t = array[j];
		while(j > 0 && array[j-1] > t){
			array[j] = array[j-1];
			j--;
		}
		array[j] = t;
	}
}

void resetForToday(AlarmPrefsType* prefs){
	UInt16			index;
	DateTimeType 	base;
	DateTimeType	oldAlarm;
	UInt32			baseSeconds;
	
	TimSecondsToDateTime(TimGetSeconds(), &base);
	if(TimDateTimeToSeconds(&prefs->alarmsStartDate) < TimDateTimeToSeconds(&base))
	{	base.hour = 0;
		base.minute = 0;	
		base.second = 0;
		baseSeconds = TimDateTimeToSeconds(&base);
	}
	else{
		base = prefs->alarmsStartDate;
	}
	for(index = 0; index < prefs->alarmsPerDay; index++){
		TimSecondsToDateTime(prefs->alarms[index], &oldAlarm);
		oldAlarm.year = base.year;
		oldAlarm.month = base.month;
		oldAlarm.day = base.day;
		prefs->alarms[index] = 	TimDateTimeToSeconds(&oldAlarm);
	}
	
	prefs->alarmsLeft = prefs->alarmsPerDay;
	for(index = 0; index < prefs->alarmsPerDay; index++){
		if(prefs->alarms[index] < TimGetSeconds()){
			prefs->alarmsLeft--;
		}	
	}
	bubleSort(prefs->alarms, prefs->alarmsPerDay);
	setAlarmPrefs(prefs);
}

void resetForTomorrow(AlarmPrefsType* prefs){
	UInt16 	index;
	UInt32	endDate = TimDateTimeToSeconds(&(prefs->alarmsEndDate));
	
	prefs->alarmsLeft = 0;
	for(index = 0; index < prefs->alarmsPerDay; index++){
		prefs->alarms[index] += SECONDSINADAY;
		prefs->alarmsLeft++;
		if(prefs->alarms[index] > endDate){
			prefs->alarms[index] = 0;
			break;
		}
	}
	setAlarmPrefs(prefs);
}

/*TODO check for research start, fix duplicate alarms*/
void randomTimes(AlarmPrefsType* prefs){
	DateTimeType	now;
	UInt32 		 	index;
	Int32			alarmWindow;
	
	TimSecondsToDateTime(TimGetSeconds(), &now);

	/*Timewindow length in seconds*/	
	alarmWindow = (prefs->alarmsEndTime.hours * 3600) 
		+ (prefs->alarmsEndTime.minutes * 60)
		- (prefs->alarmsStartTime.hours * 3600) 
		- (prefs->alarmsStartTime.minutes * 60);
	if(alarmWindow < 0){
		alarmWindow = -alarmWindow;
	}
	
	/*Random time modulo alarmwindow + alarm starttime rounded to closest minRndInterval*/
	now.hour = prefs->alarmsStartTime.hours;
	now.minute = prefs->alarmsStartTime.minutes;
	now.second = 0;
	for(index = 0; index < prefs->alarmsPerDay; index++){
		prefs->alarms[index] = TimDateTimeToSeconds(&now) + ((SysRandom(0) % alarmWindow) / 
		prefs->minRndInterval) * prefs->minRndInterval;
	}

	bubleSort(prefs->alarms, prefs->alarmsPerDay);

	/*Save*/
	setAlarmPrefs(prefs);
}

UInt32 getNextAlarm(){
	AlarmPrefsType* prefs;
	UInt32			retval = 0;
	
	prefs = getAlarmPrefs();
	
	if(true){/*theTimeisRight*/
		if(prefs->alarmsLeft > 0){
			retval = prefs->alarms[prefs->alarmsPerDay - prefs->alarmsLeft];
			prefs->alarmsLeft--;
		}
		if(prefs->alarmsLeft == 0){
			if(prefs->random){
				randomTimes(prefs);
				resetForTomorrow(prefs);
			}
			else{
				resetForTomorrow(prefs);
			}	
		}
		setAlarmPrefs(prefs);
	}
	MemPtrFree(prefs);
	/*If alarm is already gone skip to next*/
	if(retval < TimGetSeconds() && retval != 0){
		return getNextAlarm();
	}		
	return retval;
}

void setAlarmPrefs(AlarmPrefsType* prefs){

	PrefSetAppPreferences(appFileCreator, AlarmPreferences, 
		1, prefs, sizeof(AlarmPrefsType), false);	
}

void setDefaultAlarmPreferences(){
	AlarmPrefsType prefs;
	DateTimeType start;
	ErrAlert(9999);

	prefs.alarmsPerDay = 0;
	prefs.random = 0;
	prefs.alarmsLeft = 0;
	TimSecondsToDateTime(0, &start);
	prefs.alarmsStartDate = start;
	prefs.alarmsEndDate = start;
	setAlarmPrefs(&prefs);
}

AlarmPrefsType* getAlarmPrefs(){
	AlarmPrefsType*	retval;
	UInt16			prefsize;

	retval = (AlarmPrefsType*) MemPtrNew(sizeof(AlarmPrefsType));
	if(retval == NULL){
		//Out of memory	
		ErrAlert(1);
		return NULL;
	}

	prefsize = sizeof(AlarmPrefsType);
	if(PrefGetAppPreferences(appFileCreator, AlarmPreferences, retval, &prefsize, false) 
	== noPreferenceFound){
		setDefaultAlarmPreferences();
		PrefGetAppPreferences(appFileCreator, AlarmPreferences, retval, &prefsize, false);
	}

	return retval;
}