/******************************************************************************
 *
 * Copyright (c) 2007, ACCESS Systems Americas, Inc. All Rights Reserved.
 *
 * File: AppMain.c
 *
 *****************************************************************************/

#include <PalmOS.h>

#include "AppResources.h" 
#include "AlarmTimes.h"
#include "../../Preferences.h"


/***********************************************************************
 *
 *	Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *	Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			'MIHA'	// register your own at http://www.palmos.com/dev/creatorid/
#define appVersionNum			0x01
#define appPrefID				0x00
#define appPrefVersionNum		0x01

//TODO: global defines
#define sysAppLaunchCmdAlarmRet	32768
#define sysAppLaunchCmdAlarmStart 32769
#define sysAppLaunchCmdAlarmUpdate 32770

LauncherPreferenceType prefs;

/***********************************************************************
 *
 *	Internal Functions
 *
 ***********************************************************************/

void setDefaultPreferences(){
	prefs.dontExit = true;
	MemSet(prefs.password, '\0', PASSWORDMAXLENGTH + 1);
	MemSet(prefs.deviceID, '\0', DEVICEIDMAXLENGTH + 1);
	MemMove(prefs.password, "pass", 4);
	MemMove(prefs.deviceID, "0", 1);
}

/* For testing, sets alarm preferences and first alarm 60 seconds from now*/
void setAlarm(){
	AlarmPrefsType* prefs = (AlarmPrefsType*)MemPtrNew(sizeof(AlarmPrefsType));
		
	prefs->alarmsPerDay = 6;
	prefs->alarmsLeft = 6;

	TimSecondsToDateTime(TimGetSeconds(), &(prefs->alarmsStartDate)); /*Research start date*/
	TimSecondsToDateTime(TimGetSeconds() + 86400, &(prefs->alarmsEndDate)); /*Research end date*/

	/*Timewindow for alarms*/
//	prefs->alarmsStartTime.hours = 21;
//	prefs->alarmsStartTime.minutes = 40;
//	prefs->alarmsEndTime.hours = 23;
//	prefs->alarmsEndTime.minutes = 55;

	
//	prefs->random = 1;				/*If true alarmsPerDay number of random alarms */	
//	prefs->minRndInterval = 180;	/*Minimum time between random alerts(seconds)*/
//	randomTimes(prefs);				/*New random alarmtimes*/



	/*Static alarms*/
	prefs->random = 0;
	prefs->alarms[0] = TimGetSeconds() + 60;
	prefs->alarms[1] = TimGetSeconds() + 60 * 2;
	prefs->alarms[2] = TimGetSeconds() + 60 * 3;
	prefs->alarms[3] = TimGetSeconds() + 60 * 4;
	prefs->alarms[4] = TimGetSeconds() + 60 * 5;
	prefs->alarms[5] = TimGetSeconds() + 60 * 6;
	
	setAlarmPrefs(prefs);		/*Save settings*/
	resetForToday(prefs);
	MemPtrFree(prefs);

	AlmSetAlarm(0, DmFindDatabase(0, "Launcher"), NULL, getNextAlarm(), NULL); /*Set alarm*/				

}

Boolean checkPassword(){
	char* passwordEntered;
	FieldType* passField;
	
	passField = FrmGetObjectPtr(FrmGetActiveForm(), 
	FrmGetObjectIndex(FrmGetActiveForm(), PasswordField));
	passwordEntered = FldGetTextPtr(passField);
	if(strcmp(passwordEntered, prefs.password) == 0){
		return true;	
	}
	else{
		return false;
	}
}


/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
	FormType * pForm;

	switch (command) {
		case MainOptionsAboutStarterApp:
			pForm = FrmInitForm(AboutForm);
			FrmDoDialog(pForm);					// Display the About Box.
			FrmDeleteForm(pForm);
			handled = true;
			break;
			
		case MainOptionsSettings:
			pForm = FrmInitForm(PasswordForm);
			FrmGotoForm(PasswordForm);
			handled = true;
			break;
	}
	
	return handled;
}

static Boolean SettingsFormHandleEvent(EventType* pEvent)
{
	Boolean 	handled = false;
	FormType* 	pForm;
	FieldType*	passField;
	FieldType*	idField;
	char*		newPassword;
	char*		newDeviceID;
	
		
	switch (pEvent->eType) {

		case frmOpenEvent:
			pForm = FrmGetActiveForm();
			 
			passField = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm,	SettingsPWField));
			idField = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, SettingsIDField));
			
			FrmNewLabel(&pForm, 1000, prefs.password, 73, 106, stdFont);
			FrmNewLabel(&pForm, 1000, prefs.deviceID, 73, 76, stdFont);
			
			CtlSetValue((ControlType*)FrmGetObjectPtr(pForm, 
				FrmGetObjectIndex(pForm, SettingsTakeover)), prefs.dontExit);
			
			FrmDrawForm(pForm);
			handled = true;
			break;
			
		case ctlSelectEvent:
			if(pEvent->data.ctlSelect.controlID == SettingsOKButton){
				newPassword = FldGetTextPtr(passField);
				newDeviceID = FldGetTextPtr(idField);

				//Save new password and deviceID				
				if(StrLen(newPassword) > 0 && StrCompare(newPassword, "") != 0 && 
				StrLen(newPassword) <= PASSWORDMAXLENGTH){
					MemSet(prefs.password, '\0', PASSWORDMAXLENGTH + 1);
					MemMove(prefs.password, newPassword, StrLen(newPassword));
				}
				if(StrLen(newDeviceID) > 0 && StrCompare(newDeviceID, "") != 0 && 
				StrLen(newDeviceID) < DEVICEIDMAXLENGTH){
					MemSet(prefs.deviceID, '\0', DEVICEIDMAXLENGTH + 1);
					MemMove(prefs.deviceID, newDeviceID, StrLen(newDeviceID));					
				}

				FrmGotoForm(MainForm);
			}
			if(pEvent->data.ctlSelect.controlID == SettingsTakeover){
				prefs.dontExit = !prefs.dontExit;
			}
			//TODO: dontExit changes even if cancel
			if(pEvent->data.ctlSelect.controlID == SettingsCancelButton){
				FrmGotoForm(MainForm);
			}
			break;
		default:			
			break;
	}
	
	return handled;
}


static Boolean PasswordFormHandleEvent(EventType* pEvent)
{
	Boolean 	handled = false;
	FormType* 	pForm;

	switch (pEvent->eType) {

		case frmOpenEvent:
			pForm = FrmGetActiveForm();
			FrmDrawForm(pForm);
			handled = true;
			break;
			
		case ctlSelectEvent:
			if(pEvent->data.ctlSelect.controlID == PasswordOKButton){
				if(checkPassword() == true){
					FrmGotoForm(SettingsForm);
				}
				else{
					//TODO: display incorrect passwd message
					ErrAlert(1);
					FrmGotoForm(SettingsForm); // For debug
				}
			}
			if(pEvent->data.ctlSelect.controlID == PasswordCancelButton){
				FrmGotoForm(MainForm);
			}
			break;
		default:			
			break;
	}
	
	return handled;
}



/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  pEvent  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/

static Boolean MainFormHandleEvent(EventType* pEvent)
{
	Boolean 	handled = false;
	FormType* 	pForm;

	switch (pEvent->eType) {
		case menuEvent:
			return MainFormDoCommand(pEvent->data.menu.itemID);

		case frmOpenEvent:
			pForm = FrmGetActiveForm();
			FrmDrawForm(pForm);
			handled = true;
			break;
			
		default:
			break;
	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventType* pEvent)
{
	UInt16 		formId;
	FormType* 	pForm;
	Boolean		handled = false;
	LocalID     dbID;
	Err         err;

	if (pEvent->eType == frmLoadEvent) {
		// Load the form resource.
		formId = pEvent->data.frmLoad.formID;
		
		pForm = FrmInitForm(formId);
		FrmSetActiveForm(pForm);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId) {
			case MainForm:
				FrmSetEventHandler(pForm, MainFormHandleEvent);
				break;
				
			case PasswordForm:
				FrmSetEventHandler(pForm, PasswordFormHandleEvent);
				break;
				
			case SettingsForm:
				FrmSetEventHandler(pForm, SettingsFormHandleEvent);
				break;


			default:
				break;
		}
		handled = true;
	}
	
	if (pEvent->eType == ctlSelectEvent) {
		
		switch (pEvent->data.ctlSelect.controlID) {
			
			case ButtonSynchronize:	
				dbID=DmFindDatabase(0,"USBExchange");
    			if (dbID) {
    				prefs.dontExit = false;
			        err = SysUIAppSwitch(0, dbID, sysAppLaunchCmdNormalLaunch, 0);
    			}					
			break;
			
			case ButtonAnswer:
				dbID=DmFindDatabase(0,"Answer");
    			if (dbID) {
    				prefs.dontExit = false;
			        err = SysUIAppSwitch(0, dbID, sysAppLaunchCmdNormalLaunch, 0);
    			}	        
			break;
			
			default:
			break;
		}
				
	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value errNone if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
	UInt16 prefsize;
	
	//Register Upload for USB attach notification
	SysNotifyRegister(0, DmFindDatabase(0, "Upload"), sysExternalConnectorAttachEvent,
	NULL, sysNotifyNormalPriority, 0);

	//Load preferences or set defaults
	prefsize = sizeof(LauncherPreferenceType);
	if(PrefGetAppPreferences(appFileCreator, LauncherPreferences, &prefs, &prefsize, false) 
	== noPreferenceFound){
		setDefaultPreferences();
	}
/*	prefsize = sizeof(AlarmPrefsType);
	if(PrefGetAppPreferences(appFileCreator, AlarmPreferences, &prefs, &prefsize, false) 
	== noPreferenceFound){
//		TODO: Set defaults?	
	}
*/	
	FrmGotoForm(MainForm);
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{

	// Unregister upload so it wont interrup other applications	
	SysNotifyUnregister(0, DmFindDatabase(0, "Upload"), sysExternalConnectorAttachEvent,
	sysNotifyNormalPriority);
	
	// Save preferences
	PrefSetAppPreferences(appFileCreator, LauncherPreferences, 
	1, &prefs, sizeof(LauncherPreferenceType), false);
	
	// Close all the open forms.
	FrmCloseAllForms();
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	Err			error;
	EventType	event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (SysHandleEvent(&event))
			continue;
			
		if (MenuHandleEvent(0, &event, &error))
			continue;
			
		if (AppHandleEvent(&event))
			continue;

		FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent || prefs.dontExit);
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY: 
 *
 *
 ***********************************************************************/
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error = errNone;
	UInt32 temptime;

	switch (cmd) {
		case sysAppLaunchCmdNormalLaunch:
			/* Chech whether application has pending alarms*/
			if(AlmGetAlarm(0, DmFindDatabase(0, "Launcher"), NULL) == 0 || true){//always true for debug
//				Uncomment next line to test alarms
				setAlarm();
			}
			if ((error = AppStart()) == 0) {			
				AppEventLoop();
				AppStop();
			}
			break;
		/*Response to alarm goes here, open dialog, start application, set next alarm etc.*/
		case sysAppLaunchCmdDisplayAlarm:
			prefs.dontExit = false;
			SysUIAppSwitch(0, DmFindDatabase(0,"Answer"), sysAppLaunchCmdAlarmStart, 0);
			break;
		/*When answers is done after being alerted it starts Launcher with this so we know
		 * to set next alert*/
		case sysAppLaunchCmdAlarmRet:
			temptime = getNextAlarm();
			if(temptime > TimGetSeconds()){
				AlmSetAlarm(0, DmFindDatabase(0, "Launcher"), NULL, temptime, NULL);
			}
			if ((error = AppStart()) == 0) {			
				AppEventLoop();
				AppStop();
			}
		case sysAppLaunchCmdAlarmUpdate:
			resetForToday(getAlarmPrefs());
			temptime = getNextAlarm();
			if(temptime > TimGetSeconds()){
				AlmSetAlarm(0, DmFindDatabase(0, "Launcher"), NULL, temptime, NULL);
			}
			if ((error = AppStart()) == 0) {			
				AppEventLoop();
				AppStop();
			}			
		default:
			break;
	}
	
	return error;
}


