#ifndef USB_H
#define USB_H

#include <usb.h>
#include <stdio.h>
#include <time.h>
#include <string.h>

/* the device's vendor and product id */
#define MY_VID 0x830
#define MY_PID 0x61

/* the device's endpoints */
#define EP_OUT 0x4
#define EP_IN 0x82

#define BUF_SIZE 16000
#define TIMEOUT  60000
#define CONNECTION_TRIES 99999

char message_buffer[BUF_SIZE]; /* holds message data received or to be send */
usb_dev_handle *dev;           /* the usb device handle */


usb_dev_handle *open_dev(void)
{
    struct usb_bus *bus;
    struct usb_device *dev;

    for (bus = usb_get_busses(); bus; bus = bus->next) {
        for (dev = bus->devices; dev; dev = dev->next) {
            if (dev->descriptor.idVendor == MY_VID && dev->descriptor.idProduct == MY_PID) {
                /* Palm found */
                return usb_open(dev);
            }
        }
    }

    return NULL;
}

int usb_connect()
{
    dev = NULL;

    usb_init();             /* initialize the library */
    usb_find_busses();      /* find all busses */
    usb_find_devices();     /* find all connected devices */

    if (!(dev = open_dev())) {
        /* TODO ERROR: device not found*/
        return 0;
    }

    if (usb_set_configuration(dev, 1) < 0) {
        /* TODO ERROR: setting config 1 failed */
        usb_close(dev);
        return 0;
    }

    if (usb_claim_interface(dev, 0) < 0) {
        /* TODO ERROR: claiming interface 0 failed */
        usb_close(dev);
        return 0;
    }

    return 1;  /* connection established */
}

void usb_disconnect()
{
    usb_release_interface(dev, 0);
    usb_close(dev);
}

void usb_send(const char* pdata, const int pdata_size)
{  
   memset(message_buffer, '\0', BUF_SIZE);     
   memcpy(message_buffer, pdata, pdata_size);
   if (usb_bulk_write(dev, EP_OUT, message_buffer, BUF_SIZE, TIMEOUT) != BUF_SIZE) {
       /* TODO error */
   }    
}

int usb_receive()
{
    int count = 0;
    memset(message_buffer, '\0', BUF_SIZE);
    count = usb_bulk_read(dev, EP_IN, message_buffer, BUF_SIZE, TIMEOUT);
    return count;
}

const char* usb_get_message()
{
      return message_buffer;
}

#endif
