
import java.net.URL;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class StudyValidator {

    public static void main(String args[]) {
        try {
            checkArgs(args);
            SchemaFactory schemaFactory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
            Schema schema = schemaFactory.newSchema(new URL("http://kurre.googlecode.com/svn/trunk/XML/study.xsd"));
            Validator validator = schema.newValidator();
            validator.setErrorHandler(new ReportingErrorHandler());
            validator.validate(new StreamSource(args[0]));
        } catch (Exception ex) {
            System.err.println(ex.getMessage());
        }
    }

    private static void checkArgs(String args[]) {
        if (args.length != 1) {
            System.err.println("Give the filename of the XML-file to be validated!\nFor example, if file study.xml is in the same directory than this program, you can say: \"java StudyValidator study.xml\"");
            System.exit(1);
        }
    }

    private static class ReportingErrorHandler implements ErrorHandler {

        public void warning(SAXParseException exception) throws SAXException {
            System.err.println("WARNING: AT LINE " + exception.getLineNumber() + " ON COLUMN " + exception.getColumnNumber() + ":");
            System.err.println(exception.getMessage() + "\n");
        }

        public void error(SAXParseException exception) throws SAXException {
            System.err.println("ERROR: AT LINE " + exception.getLineNumber() + " ON COLUMN " + exception.getColumnNumber() + ":");
            System.err.println(exception.getMessage() + "\n");
        }

        public void fatalError(SAXParseException exception) throws SAXException {
            throw exception;
        }
    }
}

