/******************************************************************************
 *
 * Copyright (c) 2007, ACCESS Systems Americas, Inc. All Rights Reserved.
 *
 * File: AppMain.c
 *
 *****************************************************************************/

#include <PalmOS.h>
#include <StdIOPalm.h>
#include <PalmCompatibility.h>

#include <string.h>
#include <ctype.h>
#include <FileStream.h>
#include <stdlib.h>

#include "AppResources.h"
#include "CXMLReaderImpl.h"

/***********************************************************************
 *
 *	Global variables
 *
 ***********************************************************************/
int nextFreeId = 5666;
int x = 1;
int y = 50;
short maxHeight = 0;
short displayWidth, displayHeight;
FormPtr pMainForm;
//char* *gListItems; //={"Auto", "Kissa", "Kana", "Kotka", "Koira", "Puu", "Kivi", "12", "Pelto", "Takki"};
//int kListItemsCount	= 0;

char* gListItems[3];
int kListItemsCount	= 3;

char eka[256];
char toka[256];
char kolmas[256];




/***********************************************************************
 *
 *	Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *	Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			'MIHA'	// register your own at http://www.palmos.com/dev/creatorid/
#define appVersionNum			0x01
#define appPrefID				0x00
#define appPrefVersionNum		0x01

static int getNextFreeId()
{
	return nextFreeId++;
}

static void incCoordinates(short xAmount, short yAmount)
{
		x += xAmount;
		
		if (yAmount > maxHeight)
			maxHeight = yAmount;
	
	    if (x >= displayWidth) {
	    	x = 1;
	    	y += maxHeight;
	    	maxHeight = 0;
	    }
}

/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the active form.
 *
 * PARAMETERS:  objectID - id of the object
 *
 * RETURNED:    pointer to the object's data structure
 *
 ***********************************************************************/
VoidPtr GetObjectPtr(Int objectID)
{
	FormPtr frm;
	
	frm = FrmGetActiveForm();
	return(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectID)));
}

static void addCheckbox(FormPtr pForm, const char* text, int x, int y, int width)
{
	int id = getNextFreeId();
	CtlNewControl((void**)&pForm, id, checkboxCtl, text, x, y, width, 10, stdFont, 0, false);
	incCoordinates(85, 15);																															
	FrmDrawForm(pForm);
}

static void addButton(FormPtr pForm, const char* text, int x, int y)
{
	CtlNewControl((void**)&pForm, getNextFreeId(), buttonCtl, text, x, y, 0, 10, stdFont, 0, false);
	incCoordinates(40, 15);																															
	FrmDrawForm(pForm);
}

static void TestXMLParser()
{
	const char* buffer = "<root>\
	  <checkbox text=\"Kuuma?\" x=\"10\" y=\"20\" width=\"50\" />\
	  <button   text=\"Olen nappula\" x=\"93\" y=\"130\" />\
	  <checkbox text=\"Ostan jtel\" x=\"10\" y=\"35\" width=\"80\" />\
	  <checkbox text=\"Terveisi XML:lt\" x=\"10\" y=\"50\" width=\"90\" />\
	  <checkbox text=\"Kukkivatko pivnkakkarat?\" x=\"15\" y=\"90\" width=\"140\" />\
  	  <checkbox text=\"Tuuleeko pohjoisesta?\" x=\"15\" y=\"105\" width=\"140\" />\
	  <button   text=\"OK\" x=\"132\" y=\"116\" />\
	  <button   text=\"Moi vaan\" x=\"55\" y=\"70\" />\
	</root>";
	
    CBufferReadCallBack* cp = new CBufferReadCallBack(buffer, strlen(buffer));
	CXMLReaderImpl xml = CXMLReaderImpl(cp, true);
	
	while(xml.read())
	{
		switch(xml.getNodeType())
		{
		case EXN_TEXT:
			// Nothing to do here...
			break;
		case EXN_ELEMENT:
			{	
				if (!strcmp("checkbox", xml.getNodeName())) {
					addCheckbox(
						pMainForm,
						xml.getAttributeValue("text"),
						atoi(xml.getAttributeValue("x")),
						atoi(xml.getAttributeValue("y")),
						atoi(xml.getAttributeValue("width"))
					);
				}
					
				if (!strcmp("button", xml.getNodeName())) {
					addButton(
						pMainForm,
						xml.getAttributeValue("text"),
						atoi(xml.getAttributeValue("x")),
						atoi(xml.getAttributeValue("y"))
					);
				}							
			}
			break;
			
			default:
			break;
		}
	}	
}

/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
	FormType * pForm;

	switch (command) {
		case MainOptionsAboutStarterApp:
			pForm = FrmInitForm(AboutForm);
			FrmDoDialog(pForm);					// Display the About Box.
			FrmDeleteForm(pForm);
			handled = true;
			break;

	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  pEvent  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventType* pEvent)
{
	Boolean 	handled = false;
	FormType* 	pForm;

	switch (pEvent->eType) {
		case menuEvent:
			return MainFormDoCommand(pEvent->data.menu.itemID);

		case frmOpenEvent:
			pForm = FrmGetActiveForm();
			pMainForm = pForm;
			FrmDrawForm(pForm);
			handled = true;
			break;
			
		default:			
			break;
	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventType* pEvent)
{
	UInt16 		formId;
	FormType* 	pForm;
	Boolean		handled = false;

	if (pEvent->eType == frmLoadEvent) {
		// Load the form resource.
		formId = pEvent->data.frmLoad.formID;
		
		pForm = FrmInitForm(formId);
		FrmSetActiveForm(pForm);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId) {
			case MainForm:
				FrmSetEventHandler(pForm, MainFormHandleEvent);			
				break;

			default:
				break;
		}
		handled = true;
	}
	
	if (pEvent->eType == ctlSelectEvent) {

		switch (pEvent->data.ctlSelect.controlID) {			

			case ButtonReadXML:
				TestXMLParser();
			break;
			
			default:
			break;
		}
	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value errNone if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
	FrmGotoForm(MainForm);
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
	// Close all the open forms.
	FrmCloseAllForms();
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	Err			error;
	EventType	event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (SysHandleEvent(&event))
			continue;
			
		if (MenuHandleEvent(0, &event, &error))
			continue;
			
		if (AppHandleEvent(&event))
			continue;

		FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY: 
 *
 *
 ***********************************************************************/
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error = errNone;

	switch (cmd) {
		case sysAppLaunchCmdNormalLaunch:
			if ((error = AppStart()) == 0) {			
				AppEventLoop();
				AppStop();
			}
			break;

		default:
			break;
	}
	
	return error;
}


