/******************************************************************************
 *
 * Copyright (c) 2007, ACCESS Systems Americas, Inc. All Rights Reserved.
 *
 * File: AppMain.c
 *
 *****************************************************************************/

#include <PalmOS.h>
#include <PalmCompatibility.h>

#include "AppResources.h"


/***********************************************************************
 *
 *	Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *	Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			'MIHA'	// register your own at http://www.palmos.com/dev/creatorid/
#define appVersionNum			0x01
#define appPrefID				0x00
#define appPrefVersionNum		0x01
#define kListItemsCount	        10


/***********************************************************************
 *
 *	Global variables
 *
 ***********************************************************************/
int nextFreeId = 5666;
int x = 1;
int y = 50;
short maxHeight = 0;
short displayWidth, displayHeight;
FormPtr pMainForm;
char* gListItems[]={"Auto", "Kissa", "Kana", "Kotka", "Koira", "Puu", "Kivi", "12", "Pelto", "Takki"};



/***********************************************************************
 *
 *	Internal Functions
 *
 ***********************************************************************/
 static int getNextFreeId()
{
	return nextFreeId++;
}

static void incCoordinates(short xAmount, short yAmount)
{
		x += xAmount;
		
		if (yAmount > maxHeight)
			maxHeight = yAmount;
	
	    if (x >= displayWidth) {
	    	x = 1;
	    	y += maxHeight;
	    	maxHeight = 0;
	    }
}

 /***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the active form.
 *
 * PARAMETERS:  objectID - id of the object
 *
 * RETURNED:    pointer to the object's data structure
 *
 ***********************************************************************/
VoidPtr GetObjectPtr(Int objectID)
{
	FormPtr frm;
	
	frm = FrmGetActiveForm();
	return(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectID)));
}

static void addButton(FormPtr pForm)
{
	CtlNewControl((void*)&pForm, getNextFreeId(), buttonCtl, "Button", x, y, 0, 10, stdFont, 0, false);
	incCoordinates(40, 15);																															
	FrmDrawForm(pForm);
}

static void addSlider(FormPtr pForm)
{	
	CtlNewSliderControl((void*)&pForm, getNextFreeId(), sliderCtl , NULL, NULL, x, y, 30, 15, 0, 6, 1, 3);
	incCoordinates(40, 20);																															
	FrmDrawForm(pForm);
}

static void addCheckbox(FormPtr pForm)
{
	CtlNewControl((void*)&pForm, getNextFreeId(), checkboxCtl, "OK?", x, y, 35, 10, stdFont, 0, false);
	incCoordinates(40, 15);																															
	FrmDrawForm(pForm);
}

static void addLabel(FormPtr pForm)
{
	FrmNewLabel((void*)&pForm, getNextFreeId(), "Terve!", x, y, stdFont);
	incCoordinates(40, 15);																															
	FrmDrawForm(pForm);
}

static void addList(FormPtr pForm)
{
	ListPtr pList;
	int listId = getNextFreeId();
	LstNewList((void*)&pForm, listId, x, y, 35, 40, stdFont, 3, NULL);
	pList = GetObjectPtr(listId);
	
	if (pList)
		LstSetListChoices(pList, (char**)gListItems, kListItemsCount);	
		
	incCoordinates(40, 45);																															
	FrmDrawForm(pForm);
}

static void makeNoise()
{
	SndPlaySystemSound(sndAlarm);
}



/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
	FormType * pForm;

	switch (command) {
		case MainOptionsAboutStarterApp:
			pForm = FrmInitForm(AboutForm);
			FrmDoDialog(pForm);					// Display the About Box.
			FrmDeleteForm(pForm);
			handled = true;
			break;

	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  pEvent  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventType* pEvent)
{
	Boolean 	handled = false;
	FormType* 	pForm;

	switch (pEvent->eType) {
		case menuEvent:
			return MainFormDoCommand(pEvent->data.menu.itemID);

		case frmOpenEvent:
			pForm = FrmGetActiveForm();
			pMainForm = pForm;			
			FrmDrawForm(pForm);
			handled = true;
			break;
			
		default:
			break;
	}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventType* pEvent)
{
	UInt16 		formId;
	FormType* 	pForm;
	Boolean		handled = false;

	if (pEvent->eType == frmLoadEvent) {
		// Load the form resource.
		formId = pEvent->data.frmLoad.formID;
		
		pForm = FrmInitForm(formId);
		FrmSetActiveForm(pForm);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId) {
			case MainForm:
				FrmSetEventHandler(pForm, MainFormHandleEvent);
				break;

			default:
				break;
		}
		handled = true;
	}
	
	if (pEvent->eType == ctlSelectEvent) {

		switch (pEvent->data.ctlSelect.controlID) {			

			case AddButtonButton:
				addButton(pMainForm);
			break;
			
			case AddSliderButton:
				addSlider(pMainForm);
			break;
			
			case AddCheckboxButton:
				addCheckbox(pMainForm);
			break;
			
			case AddLabelButton:
				addLabel(pMainForm);
			break;
			
			case AddListButton:
				addList(pMainForm);
			break;
			
			case MakeNoiseButton:
				makeNoise();
			break;
			
			default:
			break;
		}
	}
		
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value errNone if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
	FrmGotoForm(MainForm);
	WinGetDisplayExtent(&displayWidth, &displayHeight);	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
	// Close all the open forms.
	FrmCloseAllForms();
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	Err			error;
	EventType	event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (SysHandleEvent(&event))
			continue;
			
		if (MenuHandleEvent(0, &event, &error))
			continue;
			
		if (AppHandleEvent(&event))
			continue;

		FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY: 
 *
 *
 ***********************************************************************/
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error = errNone;

	switch (cmd) {
		case sysAppLaunchCmdNormalLaunch:
			if ((error = AppStart()) == 0) {			
				AppEventLoop();
				AppStop();
			}
			break;

		default:
			break;
	}
	
	return error;
}


