/******************************************************************************
 *
 * Copyright (c) 2007, ACCESS Systems Americas, Inc. All Rights Reserved.
 *
 * File: AppMain.c
 *
 *****************************************************************************/

#include <PalmOS.h>
#include <SerialMgr.h>
#include <Window.h>
#include <SystemMgr.h>

#include "AppResources.h"


/***********************************************************************
 *
 *	Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *	Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			'MIHA'	// register your own at http://www.palmos.com/dev/creatorid/
#define appVersionNum			0x01
#define appPrefID				0x00
#define appPrefVersionNum		0x01

/***********************************************************************
 *
 *	Global variables
 *
 ***********************************************************************/
int nextFreeId = 5666;
int x = 1;
int y = 16;
short maxHeight = 0;
short displayWidth, displayHeight;
FormPtr pMainForm;
UInt16 portId;
SrmOpenConfigType config;
Err gerr;

/***********************************************************************
 *
 *	Internal Functions
 *
 ***********************************************************************/
 static int getNextFreeId()
{
	return nextFreeId++;
}

static void incCoordinates(short xAmount, short yAmount)
{
		x += xAmount;
		
		if (yAmount > maxHeight)
			maxHeight = yAmount;
	
	    if (x >= displayWidth) {
	    	x = 1;
	    	y += maxHeight;
	    	maxHeight = 0;
	    }
}

static void waitMillis(int millis)
{
	int ticksPerSec = SysTicksPerSecond();
	
	if (ticksPerSec)
		SysTaskDelay(millis * ticksPerSec / 1000);
	else
		SysTaskDelay(millis / 10);
}

static void addLabel(FormPtr pForm, char message[])
{
	FrmNewLabel(&pForm, getNextFreeId(), message, x, y, stdFont);
	incCoordinates(40, 11);																															
	FrmDrawForm(pForm);
}

static void close()
{
	if (portId) {
		int result = SrmClose(portId);
		
		switch (result)
		{
			case errNone:
				addLabel(pMainForm, "Port closed");
				portId = 0;				
				return;	
			break;
			
			case serErrBadPort:
				addLabel(pMainForm, "This port doesn't exist");
			break;
			
			case serErrNotOpen:
				addLabel(pMainForm, "The serial port is not open");
			break;
			
			case serErrNoDevicesAvail:
				addLabel(pMainForm, "No serial devices could be found");
			break;
			
			default:
			break;
		}
				
		waitMillis(500);		
		SrmClose(portId); /*Retry, it might be hard to die*/		
		portId = 0;
	}
}

/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
	FormType * pForm;

	switch (command) {
		case MainOptionsAboutStarterApp:
			pForm = FrmInitForm(AboutForm);
			FrmDoDialog(pForm);					// Display the About Box.
			FrmDeleteForm(pForm);
			handled = true;
			break;

	}
	
	return handled;
}

/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  pEvent  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventType* pEvent)
{
	Boolean 	handled = false;
	FormType* 	pForm;

	switch (pEvent->eType) {
		case menuEvent:
			return MainFormDoCommand(pEvent->data.menu.itemID);

		case frmOpenEvent:
			pForm = FrmGetActiveForm();
			pMainForm = pForm;	
			FrmDrawForm(pForm);
			handled = true;
			break;
			
		default:
			break;
	}
	
	return handled;
}

static void connect(FormPtr pForm)
{	
	int err;	
	
	close();
	
	MemSet(&config, sizeof(config), 0);
	config.function = serFncUndefined;

	err = SrmExtOpen(serPortCradleUSBPort , &config, sizeof(config), &portId); 
	
	if (err != 0)
	{		
		addLabel(pForm, "ERROR while connecting:");
		if (err == serErrBadPort) addLabel(pForm, "Connection: BAD PORT");
		if (err == serErrBadParam) addLabel(pForm, "Connection: BAD PARAM");
		if (err == serErrAlreadyOpen) addLabel(pForm, "Connection: ALREADY OPEN");
		if (err == memErrNotEnoughSpace) addLabel(pForm, "Connection: NOT ENOUGH SPACE");		
	}
}

static void sent(FormPtr pForm)
{
	UInt32 toSend, numSent; 
	Char msg[64];
	int i;
	
	for (i = 0; i < 64; i++)
		msg[i] = 'A';
	
	toSend = 64;//StrLen(msg); 
		
	if (portId) {		
		numSent = SrmSend(portId, msg, toSend, &gerr); 
		if (gerr == serErrTimeOut) { 
			addLabel(pForm, "Send: ERROR TIMEOUT");
		} 	
	}
}

static void receive(FormPtr pForm)
{
	UInt32 numReceived; 
	Char msg[64];
    MemSet(&msg, sizeof(msg), '\0');
	
	if (portId) {
		numReceived = SrmReceive(portId, msg, 64, 6000, &gerr); 
	
			switch (gerr) {
				case errNone:
				addLabel(pMainForm, "No error");
				break;
				case serErrBadPort :
				addLabel(pMainForm, "This port doesn't exist");
				break;
				case serErrNotOpen :
				addLabel(pMainForm, "The port is not open");
				break;
				case serErrTimeOut :
				addLabel(pMainForm, "Unable to receive data within the specified timeout period");
				break;
				case serErrConfigurationFailed :
				addLabel(pMainForm, "The port needs time to configure, and the configuration has failed");
				break;
				case serErrNotSupported :
				addLabel(pMainForm, "The port is not the foreground port");
				break;
				case serErrLineErr :
				addLabel(pMainForm, "A line error occurred during the receipt of data. Use SrmGetStatus() to obtain the exact line error");
				break;
				case serErrNoDevicesAvail :
				addLabel(pMainForm, "No serial devices could be found");
				break;
				default:
				break;
			}
			
			if (numReceived > 0)
				addLabel(pMainForm, msg);	
	}
}

static void doSend(FormPtr pForm)
{
	connect(pForm);
	
	addLabel(pMainForm, "Starting to send");   		

    if (portId) {
   		SrmSendFlush(portId); 	
   		SrmReceiveFlush(portId, SysTicksPerSecond() * 1.5);
    }
   
	sent(pForm); 
	
	if (portId) {
		waitMillis(1000);
	}
	
	addLabel(pForm, "Send");
	
	close();	
}

static void doReceive(FormPtr pForm)
{
	connect(pForm);
   		
	addLabel(pMainForm, "Starting to receive");   		
   
	receive(pForm); 
	
	addLabel(pForm, "Received");
	
	close();
}

/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventType* pEvent)
{
	UInt16 		formId;
	FormType* 	pForm;
	Boolean		handled = false;

	if (pEvent->eType == frmLoadEvent) {
		// Load the form resource.
		formId = pEvent->data.frmLoad.formID;
		
		pForm = FrmInitForm(formId);
		FrmSetActiveForm(pForm);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId) {
			case MainForm:
				FrmSetEventHandler(pForm, MainFormHandleEvent);
				break;

			default:
				break;
		}
		handled = true;
	}
	
	if (pEvent->eType == ctlSelectEvent) {

		switch (pEvent->data.ctlSelect.controlID) {			

			case ButtonSend:
				doSend(pMainForm);
			break;
			
			case ButtonReceive:
				doReceive(pMainForm);
			break;
									
			default:
			break;
		}
	}
		
	return handled;
}

/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value errNone if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
	FrmGotoForm(MainForm);
	return errNone;
}

/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
	// Close all the open forms.
	FrmCloseAllForms();
}

/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	Err			error;
	EventType	event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (SysHandleEvent(&event))
			continue;
			
		if (MenuHandleEvent(0, &event, &error))
			continue;
			
		if (AppHandleEvent(&event))
			continue;

		FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}

/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY: 
 *
 *
 ***********************************************************************/
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error = errNone;

	switch (cmd) {
		case sysAppLaunchCmdNormalLaunch:
			if ((error = AppStart()) == 0) {			
				AppEventLoop();
				AppStop();
			}
			break;

		default:
			break;
	}
	
	return error;
}
