#include <usb.h>
#include <stdio.h>

/* the device's vendor and product id */
#define MY_VID 0x830
#define MY_PID 0x61

/* the device's endpoint(s) */
#define EP_IN 0x82

#define BUF_SIZE 64
#define TIMEOUT  7000

usb_dev_handle *open_dev(void)
{
    struct usb_bus *bus;
    struct usb_device *dev;

    for (bus = usb_get_busses(); bus; bus = bus->next) {
        for(dev = bus->devices; dev; dev = dev->next)  {
            if (dev->descriptor.idVendor == MY_VID && dev->descriptor.idProduct == MY_PID) {
                printf("Palm found\n");
                return usb_open(dev);
            }
        }
    }

    return NULL;
}

int main(void)
{
    usb_dev_handle *dev = NULL; /* the device handle */
    char tmp[BUF_SIZE];
    int i;

    usb_init();         /* initialize the library */
    usb_find_busses();  /* find all busses */
    usb_find_devices(); /* find all connected devices */

    if (!(dev = open_dev())) {
        printf("ERROR: device not found!\n");
        return 0;
    }

    if (usb_set_configuration(dev, 1) < 0) {
        printf("error: setting config 1 failed\n");
        usb_close(dev);
        return 0;
    }

    if (usb_claim_interface(dev, 0) < 0) {
        printf("error: claiming interface 0 failed\n");
        usb_close(dev);
        return 0;
    }

    int count = usb_bulk_read(dev, EP_IN, tmp, sizeof(tmp), TIMEOUT);
    if (count <= 0) {
        printf("ERROR: bulk read failed %d %s\n", count, usb_strerror());
    } else {
        for (i = 0; i < count; ++i)
            printf("%c", tmp[i]);

        printf("\n");
    }

    usb_release_interface(dev, 0);
    usb_close(dev);

    return 0;
}
